/*  Copyright (c) 2011-2016  Milos Jakubicek

Obtained from http://unicode.org/repos/cldr/trunk/common/supplemental/likelySubtags.xml

by the following:

grep '<likelySubtag ' likelySubtags.xml | grep -v und |
sed 's#.* to="\([^"_]\+\)_\([^"_]\+_\)\?\([^"_]\+\)"/> <!--{ \([^;]\+\).*#{"\4", "\1_\3"},#'

and modified manually:

{"Chinese", "zh_CN"},
{"Chinese Simplified", "zh_CN"},
{"Chinese Traditional", "zh_TW"},
{"Setswana", "tn_ZA"},
{"Frisian", "fy_NL"},
{"Maldivian", "dv_MV"},
{"Norwegian", "no_NO"},
{"Yiddish", "yi_US"},
{"Kyrgyz", "ky_KG"},
{"Esperanto", "C"},
{"-- other (UTF-8) --"] = "C"},
*/

#include <config.hh>
#include <string>
#include <cstdio>
#include <cstring>
#include <unordered_map>
#include <functional>

using namespace std;
typedef unordered_map <string, const string> ss_map;

static const ss_map langcodes({
    {"Afar", "aa_ET"},
    {"Abkhazian", "ab_GE"},
    {"Adyghe", "ady_RU"},
    {"Afrikaans", "af_ZA"},
    {"Aghem", "agq_CM"},
    {"Akan", "ak_GH"},
    {"Amharic", "am_ET"},
    {"Arabic", "ar_EG"},
    {"Assamese", "as_IN"},
    {"Asu", "asa_TZ"},
    {"Asturian", "ast_ES"},
    {"Avaric", "av_RU"},
    {"Aymara", "ay_BO"},
    {"Azerbaijani", "az_AZ"},
    {"Azerbaijani", "az_IR"},
    {"Azerbaijani", "az_IR"},
    {"Bashkir", "ba_RU"},
    {"Basaa", "bas_CM"},
    {"Belarusian", "be_BY"},
    {"Bemba", "bem_ZM"},
    {"Bena", "bez_TZ"},
    {"Bulgarian", "bg_BG"},
    {"Bislama", "bi_VU"},
    {"Bambara", "bm_ML"},
    {"Bengali", "bn_BD"},
    {"Tibetan", "bo_CN"},
    {"Breton", "br_FR"},
    {"Bodo", "brx_IN"},
    {"Bosnian", "bs_BA"},
    {"Blin", "byn_ER"},
    {"Catalan", "ca_ES"},
    {"Atsam", "cch_NG"},
    {"Chechen", "ce_RU"},
    {"Cebuano", "ceb_PH"},
    {"Chiga", "cgg_UG"},
    {"Chamorro", "ch_GU"},
    {"Chuukese", "chk_FM"},
    {"Cherokee", "chr_US"},
    {"Czech", "cs_CZ"},
    {"Welsh", "cy_GB"},
    {"Danish", "da_DK"},
    {"Taita", "dav_KE"},
    {"German", "de_DE"},
    {"Zarma", "dje_NE"},
    {"Duala", "dua_CM"},
    {"Divehi", "dv_MV"},
    {"Maldivian", "dv_MV"},
    {"Jola-Fonyi", "dyo_SN"},
    {"Dzongkha", "dz_BT"},
    {"Embu", "ebu_KE"},
    {"Ewe", "ee_GH"},
    {"Efik", "efi_NG"},
    {"Greek", "el_GR"},
    {"English", "en_US"},
    {"Spanish", "es_ES"},
    {"Estonian", "et_EE"},
    {"Basque", "eu_ES"},
    {"Ewondo", "ewo_CM"},
    {"Persian", "fa_IR"},
    {"Fulah", "ff_SN"},
    {"Finnish", "fi_FI"},
    {"Filipino", "fil_PH"},
    {"Fijian", "fj_FJ"},
    {"Faroese", "fo_FO"},
    {"French", "fr_FR"},
    {"Friulian", "fur_IT"},
    {"Frisian", "fy_NL"},
    {"Western Frisian", "fy_NL"},
    {"Irish", "ga_IE"},
    {"Ga", "gaa_GH"},
    {"Gagauz", "gag_MD"},
    {"Scottish Gaelic", "gd_GB"},
    {"Gilbertese", "gil_KI"},
    {"Galician", "gl_ES"},
    {"Guarani", "gn_PY"},
    {"Swiss German", "gsw_CH"},
    {"Gujarati", "gu_IN"},
    {"Gusii", "guz_KE"},
    {"Manx", "gv_GB"},
    {"Hausa", "ha_NG"},
    {"Hawaiian", "haw_US"},
    {"Hebrew", "he_IL"},
    {"Hindi", "hi_IN"},
    {"Hiligaynon", "hil_PH"},
    {"Hiri Motu", "ho_PG"},
    {"Croatian", "hr_HR"},
    {"Haitian", "ht_HT"},
    {"Hungarian", "hu_HU"},
    {"Armenian", "hy_AM"},
    {"Indonesian", "id_ID"},
    {"Igbo", "ig_NG"},
    {"Sichuan Yi", "ii_CN"},
    {"Iloko", "ilo_PH"},
    {"Ingush", "inh_RU"},
    {"Icelandic", "is_IS"},
    {"Italian", "it_IT"},
    {"Japanese", "ja_JP"},
    {"Machame", "jmc_TZ"},
    {"Javanese", "jv_ID"},
    {"Georgian", "ka_GE"},
    {"Kabyle", "kab_DZ"},
    {"Jju", "kaj_NG"},
    {"Kamba", "kam_KE"},
    {"Kabardian", "kbd_RU"},
    {"Tyap", "kcg_NG"},
    {"Makonde", "kde_TZ"},
    {"Kabuverdianu", "kea_CV"},
    {"Kongo", "kg_CD"},
    {"Khasi", "kha_IN"},
    {"Koyra Chiini", "khq_ML"},
    {"Kikuyu", "ki_KE"},
    {"Kuanyama", "kj_NA"},
    {"Kazakh", "kk_KZ"},
    {"Kalaallisut", "kl_GL"},
    {"Kalenjin", "kln_KE"},
    {"Khmer", "km_KH"},
    {"Kannada", "kn_IN"},
    {"Korean", "ko_KR"},
    {"Komi-Permyak", "koi_RU"},
    {"Konkani", "kok_IN"},
    {"Kosraean", "kos_FM"},
    {"Kpelle", "kpe_LR"},
    {"Komi-Zyrian", "kpv_RU"},
    {"Karachay-Balkar", "krc_RU"},
    {"Kashmiri", "ks_IN"},
    {"Shambala", "ksb_TZ"},
    {"Bafia", "ksf_CM"},
    {"Colognian", "ksh_DE"},
    {"Kurdish", "ku_IQ"},
    {"Kurdish", "ku_TR"},
    {"Kurdish", "ku_SY"},
    {"Kurdish", "ku_TR"},
    {"Kumyk", "kum_RU"},
    {"Cornish", "kw_GB"},
    {"Kirghiz", "ky_KG"},
    {"Kyrgyz", "ky_KG"},
    {"Latin", "la_VA"},
    {"Langi", "lag_TZ"},
    {"Lahnda", "lah_PK"},
    {"Luxembourgish", "lb_LU"},
    {"Lak", "lbe_RU"},
    {"Lezghian", "lez_RU"},
    {"Ganda", "lg_UG"},
    {"Lingala", "ln_CD"},
    {"Lao", "lo_LA"},
    {"Limburgish", "li_NL"},
    {"Lithuanian", "lt_LT"},
    {"Luba-Katanga", "lu_CD"},
    {"Luba-Lulua", "lua_CD"},
    {"Luo", "luo_KE"},
    {"Luyia", "luy_KE"},
    {"Latvian", "lv_LV"},
    {"Maithili", "mai_IN"},
    {"Masai", "mas_KE"},
    {"Moksha", "mdf_RU"},
    {"Maguindanaon", "mdh_PH"},
    {"Meru", "mer_KE"},
    {"Morisyen", "mfe_MU"},
    {"Malagasy", "mg_MG"},
    {"Makhuwa-Meetto", "mgh_MZ"},
    {"Marshallese", "mh_MH"},
    {"Maori", "mi_NZ"},
    {"Macedonian", "mk_MK"},
    {"Malayalam", "ml_IN"},
    {"Mongolian", "mn_MN"},
    {"Mongolian", "mn_CN"},
    {"Mongolian", "mn_CN"},
    {"Marathi", "mr_IN"},
    {"Malay", "ms_MY"},
    {"Maltese", "mt_MT"},
    {"Burmese", "my_MM"},
    {"Erzya", "myv_RU"},
    {"Nauru", "na_NR"},
    {"Nama", "naq_NA"},
    {"Norwegian", "no_NO"},
    {"Norwegian Bokmål", "nb_NO"},
    {"North Ndebele", "nd_ZW"},
    {"Low German", "nds_DE"},
    {"Nepali", "ne_NP"},
    {"Niuean", "niu_NU"},
    {"Dutch", "nl_NL"},
    {"Kwasio", "nmg_CM"},
    {"Norwegian Nynorsk", "nn_NO"},
    {"South Ndebele", "nr_ZA"},
    {"Northern Sotho", "nso_ZA"},
    {"Nuer", "nus_SD"},
    {"Nyanja", "ny_MW"},
    {"Nyankole", "nyn_UG"},
    {"Occitan", "oc_FR"},
    {"Oromo", "om_ET"},
    {"Oriya", "or_IN"},
    {"Ossetic", "os_GE"},
    {"Punjabi", "pa_IN"},
    {"Punjabi", "pa_PK"},
    {"Punjabi", "pa_PK"},
    {"Pangasinan", "pag_PH"},
    {"Papiamento", "pap_AN"},
    {"Palauan", "pau_PW"},
    {"Polish", "pl_PL"},
    {"Pohnpeian", "pon_FM"},
    {"Pashto", "ps_AF"},
    {"Portuguese", "pt_BR"},
    {"Quechua", "qu_PE"},
    {"Romansh", "rm_CH"},
    {"Romanian", "ro_RO"},
    {"Rombo", "rof_TZ"},
    {"Russian", "ru_RU"},
    {"Kinyarwanda", "rw_RW"},
    {"Rwa", "rwk_TZ"},
    {"Sanskrit", "sa_IN"},
    {"Sakha", "sah_RU"},
    {"Samburu", "saq_KE"},
    {"Santali", "sat_IN"},
    {"Sangu", "sbp_TZ"},
    {"Sindhi", "sd_IN"},
    {"Northern Sami", "se_NO"},
    {"Sena", "seh_MZ"},
    {"Koyraboro Senni", "ses_ML"},
    {"Sango", "sg_CF"},
    {"Tachelhit", "shi_MA"},
    {"Sinhala", "si_LK"},
    {"Sidamo", "sid_ET"},
    {"Slovak", "sk_SK"},
    {"Slovenian", "sl_SI"},
    {"Samoan", "sm_WS"},
    {"Shona", "sn_ZW"},
    {"Somali", "so_SO"},
    {"Albanian", "sq_AL"},
    {"Serbian", "sr_RS"},
    {"Serbian", "sr_RS"},
    {"Serbian", "sr_ME"},
    {"Setswana", "tn_ZA"},
    {"Swati", "ss_ZA"},
    {"Saho", "ssy_ER"},
    {"Southern Sotho", "st_ZA"},
    {"Swedish", "sv_SE"},
    {"Swahili", "sw_TZ"},
    {"Congo Swahili", "swc_CD"},
    {"Tamil", "ta_IN"},
    {"Telugu", "te_IN"},
    {"Teso", "teo_UG"},
    {"Tetum", "tet_TL"},
    {"Tajik", "tg_TJ"},
    {"Thai", "th_TH"},
    {"Tigrinya", "ti_ET"},
    {"Tigre", "tig_ER"},
    {"Turkmen", "tk_TM"},
    {"Tokelau", "tkl_TK"},
    {"Tswana", "tn_ZA"},
    {"Tonga", "to_TO"},
    {"Tok Pisin", "tpi_PG"},
    {"Turkish", "tr_TR"},
    {"Taroko", "trv_TW"},
    {"Tsonga", "ts_ZA"},
    {"Tausug", "tsg_PH"},
    {"Tatar", "tt_RU"},
    {"Tuvalu", "tvl_TV"},
    {"Tasawaq", "twq_NE"},
    {"Tahitian", "ty_PF"},
    {"Tuvinian", "tyv_RU"},
    {"Central Morocco Tamazight", "tzm_MA"},
    {"Udmurt", "udm_RU"},
    {"Uighur", "ug_CN"},
    {"Ukrainian", "uk_UA"},
    {"Ulithian", "uli_FM"},
    {"Urdu", "ur_PK"},
    {"Uzbek", "uz_UZ"},
    {"Uzbek", "uz_AF"},
    {"Uzbek", "uz_AF"},
    {"Vai", "vai_LR"},
    {"Venda", "ve_ZA"},
    {"Vietnamese", "vi_VN"},
    {"Vunjo", "vun_TZ"},
    {"Walser", "wae_CH"},
    {"Walamo", "wal_ET"},
    {"Waray", "war_PH"},
    {"Wolof", "wo_SN"},
    {"Xhosa", "xh_ZA"},
    {"Soga", "xog_UG"},
    {"Yapese", "yap_FM"},
    {"Yangben", "yav_CM"},
    {"Yiddish", "yi_US"},
    {"Yoruba", "yo_NG"},
    {"Zhuang", "za_CN"},
    {"Chinese", "zh_CN"},
    {"Chinese Simplified", "zh_CN"},
    {"Chinese Traditional", "zh_TW"},
    {"Zulu", "zu_ZA"},
    {"Esperanto", "C"},
    {"-- other (UTF-8) --", "C"}
});

void languages (vector<string> &out) {
    for (ss_map::const_iterator i = langcodes.cbegin(); i != langcodes.cend(); i++)
        out.push_back (i->first);
}

// vim: ts=4 sw=4 sta et sts=4 si cindent tw=80:
